const mysql = require('mysql')

/**
 * @class Mysql
 */
class Mysql {

  constructor (configuration) {
    this.pool = mysql.createPool(configuration)
    this.connection = null
  }

  /**
   * @function getConnection
   * @description This function returns the current used connection that can be used
   * to query the database.
   */
  getConnection () {
    return new Promise((resolve, reject) => {
      this.pool.getConnection((err, connection) => {
        if (err) {
          reject(err)
          return
        }

        resolve(connection)
      })
    })
  }

  /**
   * @function getConnectionAndQuery
   * @description This function takes a new connection, runs a query and returns the
   * query result. 
   * @param {string} query 
   */
  getConnectionAndQuery (query) {
    return new Promise((resolve, reject) => {
      this.getConnection()
        .then((connection) => {
          connection.query(query, (err, results, fields) => {
            if (err) {
              reject(err)
              return
            }
      
            resolve(results, fields)
          })
        })
        .catch((error) => {
          reject(error)
        })
    })
  }

  /**
   * @function startQueries
   */
  startQueries () {
    return new Promise((resolve, reject) => {
      this.getConnection()
        .then((connection) => {
          this.connection = connection
          resolve()
        })
        .catch((error) => {
          reject(error)
        })
    })
  }

  /**
   * @function stopQueries
   */
  stopQueries () {
    this.connection = null
  }

  /**
   * @function query
   * @param {string} query 
   */
  query (query) {
    return new Promise((resolve, reject) => {
      if (!this.connection) {
        reject(new Error('connection can not be null'))
        return
      }

      this.connection.query(query, (err, results, fields) => {
        if (err) {
          reject(err)
          return
        }
  
        resolve(results, fields)
      })
    })
  }

  /**
   * @function closePool
   * @description This function closes the current pool.
   */
  closePool () {
    return new Promise((resolve, reject) => {
      this.pool.end((err) => {
        if (err) {
          reject(err)
          return
        }

        this.pool = null
        resolve()
      })
    })
  }

}

module.exports = Mysql
